import './JSXColoringProvider.css';

let monaco = null, j = null;

const defaultOptions = {
    isHighlightGlyph: true,
    iShowHover: false,
};

const HIGHLIGHT_SCOPE = {
    ALL: 'ALL',
    IDENTIFIER: 'IDENTIFIER',
    LOCAL: 'LOCAL',
    EXTRA: 'EXTRA',
};

export const JSXTypes = {
    JSXBracket: {
        highlightScope: HIGHLIGHT_SCOPE.LOCAL,
        options: {
            inlineClassName: 'mtk100.Identifier.JsxElement.Bracket',
        },
    },
    JSXOpeningElement: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk101.Identifier.JsxOpeningElement.Identifier',
        },
    },
    JSXClosingElement: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk102.Identifier.JsxClosingElement.Identifier ',
        },
    },
    JSXAttribute: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk103.Identifier.JsxAttribute.Identifier ',
        },
    },
    JSXText: {
        highlightScope: HIGHLIGHT_SCOPE.ALL,
        options: {
            inlineClassName: 'mtk104.JsxElement.JsxText',
        },
    },
    JSXElement: {
        highlightScope: HIGHLIGHT_SCOPE.EXTRA,
        options: (elementName) => (
            {
                glyphMarginClassName: 'mtk105.glyph.Identifier.JsxElement',
                glyphMarginHoverMessage:
                    `JSX Element${elementName ? ': ' + elementName : ''}`
            }
        ),
    },
};

export const configureLocToMonacoRange = (_monaco = monaco, parser = 'babylon') => {
    switch (parser) {
        case 'babylon':
        default:
            return loc => {
                if (!loc || !loc.start) {
                    return new _monaco.Range(1, 1, 1, 1);
                }
                return new _monaco.Range(loc.start.line
                    , loc.start.column + 1
                    , loc.end ? loc.end.line : loc.start.line
                    , loc.end ? loc.end.column + 1 : loc.start.column + 1,
                );
            };
    }
};

class JSXColoringProvider {
    constructor(
        monacoRef,
        jRef,
        monacoEditor,
        options = defaultOptions,
        editorId
    ) {
        monaco = monacoRef;
        j = jRef;
        this.locToMonacoRange = configureLocToMonacoRange(monaco);
        this.monacoEditor = monacoEditor;
        this.options = options;
        this.editorId = editorId;
    }

    highLightOnDidChangeModelContent(
        onResolve = ast => ast,
        onError = error => console.error(error),
    ) {
        const {monacoEditor, colorizeCode, options} = this;
        colorizeCode(options);
        let highlighterDisposer = monacoEditor.onDidChangeModelContent(
            () => this.colorizeCode(options)
                .then(onResolve)
                .catch(onError)
        );
        return () => {
            if (!highlighterDisposer) {
                return;
            }
            highlighterDisposer.dispose();
            monacoEditor.deltaDecorations(
                this.JSXDecoratorIds || [],
                [],
            );
            highlighterDisposer = null;
        };
    }

    colorize(source, isSourceCode, options) {
        return new Promise((resolve) => {
            const ast = isSourceCode ? j(source) : source;
            if (ast) {
                const decorators = this.createJSXElementDecorators(ast, options);
                for (const jsxType in JSXTypes) {
                    this.createDecoratorsByType(
                        ast,
                        jsxType,
                        JSXTypes[jsxType].options,
                        decorators,
                        options,
                    );
                }
                this.JSXDecoratorIds =
                    this.monacoEditor.deltaDecorations(
                        this.JSXDecoratorIds || [],
                        decorators,
                    );
                resolve(ast);
            }
        });

    }

    colorizeCode(options) {
        this.colorize(this.monacoEditor.getValue(), true, options);
    }

    createJSXElementDecorators(
        ast,
        decorators = [],
        options = this.options,
    ) {
        ast
            .findJSXElements()
            .forEach(p => {
                const loc = p.value.loc;
                const openingElement = p.value.openingElement;
                let elementName = null;
                if (openingElement) {
                    const oLoc = openingElement.loc;
                    elementName = openingElement.name.name;
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.start.line,
                            oLoc.start.column + 1,
                            oLoc.start.line,
                            oLoc.start.column + 2
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.end.line,
                            oLoc.end.column + (openingElement.selfClosing ? -1 : 0),
                            oLoc.end.line,
                            oLoc.end.column + 1
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                }
                const closingElement = p.value.closingElement;
                if (closingElement) {
                    const cLoc = closingElement.loc;
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.start.line,
                            cLoc.start.column + 1,
                            cLoc.start.line,
                            cLoc.start.column + 3
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.end.line,
                            cLoc.end.column,
                            cLoc.end.line,
                            cLoc.end.column + 1
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                }

                options.isHighlightGlyph && decorators.push({
                    range: this.locToMonacoRange(loc),
                    options: JSXTypes.JSXElement.options(elementName),
                });
            });
        return decorators;
    }

    createDecoratorsByType(
        ast,
        jsxType,
        jsxTypeOptions,
        decorators = [],
        highlighterOptions = this.options,
    ) {
        ast.find(j[jsxType].filter(conf => conf.highlightScope.IDENTIFIER))
            .find(j.JSXIdentifier)
            .forEach(p => {
                const loc = p.value.loc;
                const options = highlighterOptions.iShowHover ?
                    {...jsxTypeOptions, ...{hoverMessage: `(${jsxType})`}}
                    : jsxTypeOptions;
                decorators.push({
                    range: this.locToMonacoRange(loc),
                    options
                });
            });
        ast.find(j[jsxType].filter(conf => conf.highlightScope.ALL))
            .forEach(p => {
                const loc = p.value.loc;
                const options = highlighterOptions.iShowHover ?
                    {...jsxTypeOptions, ...{hoverMessage: `(${jsxType})`}}
                    : jsxTypeOptions;
                decorators.push({
                    range: this.locToMonacoRange(loc),
                    options
                });
            });
        return decorators;
    }
}

export default JSXColoringProvider;